"""
This module provides functions for the vegmod package.
"""
import logging
import os
from dotenv import load_dotenv
import praw

load_dotenv()

handler = logging.StreamHandler()
handler.setLevel(logging.DEBUG)
for logger_name in ("praw", "prawcore"):
    logger = logging.getLogger(logger_name)
    logger.setLevel(logging.DEBUG)
    logger.addHandler(handler)

reddit = praw.Reddit(
    client_id=os.environ.get("REDDIT_CLIENT_ID"),
    client_secret=os.environ.get("REDDIT_CLIENT_SECRET"),
    user_agent=os.environ.get("REDDIT_USER_AGENT"),
    username=os.environ.get("REDDIT_USERNAME"),
    password=os.environ.get("REDDIT_PASSWORD"),
)



import json
import re
import os
import typer
import vegmod.ingress
from vegmod import reddit
from time import sleep

from vegmod.pycall import subreddit_widgets_sidebar_delete_all, subreddit_widgets_mod_add_image_widget, subreddit_widgets_mod_add_button_widget, subreddit_widgets_mod_add_community_list

app = typer.Typer()

@app.command()
def ingress():
    """
    Pull data from the subreddits and save it to a JSON file.
    """
    subreddits = os.environ.get("INGRESS_SUBREDDITS").split(",")
    typer.echo(f"Pulling data from {subreddits}")
    vegmod.ingress.pull([reddit.subreddit(subreddit) for subreddit in subreddits])
    typer.echo(f"Data pulled and saved to {vegmod.ingress.INGRESS_FILE_PATH}")

@app.command()
def ls():
    """
    Lists the subreddits that will be pulled from.
    """
    subreddits = os.environ.get("INGRESS_SUBREDDITS").split(",")
    for subreddit in subreddits:
        typer.echo(subreddit)
        
@app.command()
def setIcon(subreddit: str, icon_path: str):
    """
    Set the icon for a subreddit.
    """
    community_icon_url = reddit.subreddit(subreddit).stylesheet._upload_style_asset(
        image_path=icon_path,
        image_type="communityIcon",
    )
    
    reddit.subreddit(subreddit).stylesheet._update_structured_styles(
    {
        'communityIcon': community_icon_url
    })

@app.command()
def inspect(object_type: str, object_id: str):
    """
    Object types: submission, comment, subreddit, redditor
    """
    if object_type == "submission":
        obj = reddit.submission(object_id)
        # t = obj.title
    elif object_type == "comment":
        obj = reddit.comment(object_id)
        typer.echo("comment vars")
        typer.echo(vars(obj))
        
        author = obj.author
        typer.echo("comment author vars")
        typer.echo(vars(author))
        
        typer.echo(f"comment author vars expanded {author.comment_karma}")
        typer.echo(vars(author))
        # t = obj.body
    elif object_type == "subreddit":
        obj = reddit.subreddit(object_id)
        typer.echo("subreddit vars")
        typer.echo(vars(obj))
        
        rules = obj.rules
        typer.echo("subreddit rules vars")
        typer.echo(vars(rules))
        
        typer.echo("subreddit rules printed")
        typer.echo(rules)
        
        for rule in rules:
            typer.echo("-------------------------")
            typer.echo(rule.created_utc)
            typer.echo(rule.description)
            typer.echo(rule.kind)
            typer.echo(rule.priority)
            typer.echo(rule.short_name)
            typer.echo(rule.violation_reason)
            typer.echo("====== vars ======")
            typer.echo(vars(rule))
            typer.echo("-------------------------")
            
        # t = obj.body
    elif object_type == "redditor":
        obj = reddit.redditor(object_id)
        # t = obj.comment_karma
    else:
        raise ValueError(f"Invalid object type: {object_type}, must be one of: submission, comment, subreddit, user")
    # vars_string = str(vars(obj))
    
    # typer.echo("-vars_string-------------")
    # typer.echo(vars_string)
    # typer.echo("-------------------------")
    
    # # python objects are represented as strings in the vars() output
    # # '_reddit': <praw.reddit.Reddit object at 0x735720051f70>,
    # # so we need to convert them to
    # # '_reddit': "<praw.reddit.Reddit object at 0x735720051f70>",
    # json_string = re.sub(r'(<[a-zA-Z0-9._\s]+object[a-zA-Z0-9._\s]+>)', r'"\1"', vars_string)
    
    # # also UserSubreddit(display_name='vegmod'),
    # # should be converted to
    # # '<UserSubreddit>',
    # json_string = re.sub(r'(\s)([A-Z])([a-zA-Z]+)\(([^\)]+)\)', r'\1"<\2\3>"', json_string)
    
    # # replace all single quotes that are surrounded by double quotes with &apos;
    # # this is necessary because the JSON module does not support single quotes
    # # inside of double quotes
    # in_double_quotes = False
    # in_single_quotes = False
    # new_json_string = ""
    # for i, c in enumerate(json_string):
    #     if c == "'" and in_double_quotes:
    #         new_json_string += "&apos;"
    #         continue
    #     if c == '"' and in_single_quotes:
    #         new_json_string += "&quot;"
    #         continue
    #     if c == '"':
    #         in_double_quotes = not in_double_quotes
    #     if c == "'":
    #         in_single_quotes = not in_single_quotes
    #     new_json_string += c
    # json_string = new_json_string

    # json_string = json_string.replace("'", '"')
    # json_string = json_string.replace("&apos;", "'")
        
    # json_string = json_string.replace("None", "null").\
    #     replace("True", "true").\
    #     replace("False", "false")
        
    # typer.echo("-json_string-------------")
    # typer.echo(json_string)
    # typer.echo("-------------------------")
    
    # json_obj = json.loads(json_string)

    # typer.echo(json.dumps(json_obj, indent=4))
    
@app.command()
def test_widget_configuration(subreddit: str):
    test_img_url = "https://i.redd.it/msxd2or7psld1.jpeg"
    text_img_link_url = "https://www.reddit.com/r/vegancirclejerk/comments/1f4u1xd/carnist_has_a_son/"
    test_communities = ['veganlobby', 'veganvets']
    test_texts = ['Vegmod', 'Veganism Social']
    test_urls = ['https://vegmod.com', 'https://veganism.social']
    
    subreddit_widgets_sidebar_delete_all(subreddit)
    subreddit_widgets_mod_add_image_widget(subreddit, "short name (img 12)", test_img_url, text_img_link_url)
    subreddit_widgets_mod_add_community_list(subreddit, "short name (com 12)", "description", test_communities)
    subreddit_widgets_mod_add_button_widget(subreddit, "short name (but 12)", "description", test_texts, test_urls)

@app.command()
def image_widgets(subreddit: str):
    widgets = reddit.subreddit(subreddit).widgets.sidebar
    for widget in widgets:
        if widget.kind == "image":
            typer.echo("widget.data='{}'".format(widget.data))
            typer.echo("widget.id='{}'".format(widget.id))
            typer.echo("widget.kind='{}'".format(widget.kind))
            typer.echo("widget.shortName='{}'".format(widget.shortName))
            typer.echo("widget.styles='{}'".format(widget.styles))
            typer.echo("widget.subreddit='{}'".format(widget.subreddit))
            
            index = 0
            for image in widget:
                typer.echo(f"image INDEX={index}")
                typer.echo("image.url='{}'".format(image.url))
                typer.echo("image.linkUrl='{}'".format(image.linkUrl))
                typer.echo("image.width='{}'".format(image.width))
                typer.echo("image.height='{}'".format(image.height))
                index += 1

@app.command()
def refresh_widgets(subreddit: str):
    reddit.subreddit(subreddit).widgets.refresh()

def main():
    app()



import os
import time
from loguru import logger
import praw
from vegmod.serializer import serialize, serialize_list
from vegmod.utils import save_dict
from vegmod.cache import Cache

DATA_DIR = f"{os.path.dirname(__file__)}/../data"
INGRESS_FILE_PATH=f"{DATA_DIR}/ingress.json"
CACHE_FILE_PATH=f"{DATA_DIR}/ingress_cache.json"
REQUEST_DELAY = 1

def pull(subreddits: list[praw.models.Subreddit]):
    """
    Pull data from the subreddits and save it to a JSON file.
    """
    data = {}
    for subreddit in subreddits:
        cache = Cache(CACHE_FILE_PATH)

        logger.info(f"Pulling subreddit={subreddit.display_name}")
        subreddit_data = serialize(subreddit, cache=cache)
        time.sleep(REQUEST_DELAY)

        logger.info(f"Pulling subreddit={subreddit.display_name} submissions")
        submissions = list(subreddit.new(limit=25))
        subreddit_data["submissions"] = serialize_list(submissions, cache=cache)
        time.sleep(REQUEST_DELAY)

        logger.info(f"Pulling subreddit={subreddit.display_name} comments")
        comments = list(subreddit.comments(limit=100)) # 100 gives longer score updates
        subreddit_data["comments"] = serialize_list(comments, cache=cache)
        time.sleep(REQUEST_DELAY)

        logger.info(f"Pulling subreddit={subreddit.display_name} removal reasons")
        removal_reasons = list(subreddit.mod.removal_reasons)
        subreddit_data["removal_reasons"] = serialize_list(removal_reasons, cache=cache)
        time.sleep(REQUEST_DELAY)

        logger.info(f"Pulling subreddit={subreddit.display_name} rules")
        rules = list(subreddit.rules)
        subreddit_data["rules"] = serialize_list(rules, cache=cache)
        time.sleep(REQUEST_DELAY)
        
        logger.info(f"Pulling subreddit={subreddit.display_name} widgets.sidebar")
        widgets_sidebar = list(subreddit.widgets.sidebar)
        subreddit_data["widgets_sidebar"] = serialize_list(widgets_sidebar, cache=cache)
        time.sleep(REQUEST_DELAY)

        data[subreddit.display_name] = subreddit_data
        cache.save()

    save_dict(data, INGRESS_FILE_PATH)



"""
This module contains functions for serializing PRAW objects into JSON-serializable dictionaries.
"""
import time
import praw
from loguru import logger
import traceback
from vegmod.cache import Cache

def serialize(o, cache : Cache) -> dict:
    """
    Serialize a PRAW object into a JSON-serializable dictionary.
    """
    if o is None:
        return None

    s = {}
    try:
        s = _switch_case(o, cache=cache)
    except Exception as e:
        logger.error(f"Failed to serialize object={o} type={type(o).__name__} error={e}")
        traceback.print_exc()
        s["_error"] = str(e)
        s["_error_type"] = type(e).__name__
    finally:
        s["_type"] = type(o).__name__
        s["_serialized_at"] = time.time()

    return s

def serialize_list(o: list, cache : Cache = None) -> list:
    """
    Serialize a list of PRAW objects into a list of JSON-serializable dictionaries.
    """
    data = []
    for item in o:
        data.append(
            serialize(item, cache=cache)
        )
    return data

def _switch_case(obj, cache : Cache) -> dict:
    """
    Serialize a PRAW object into a JSON-serializable dictionary using a switch-case pattern.
    """
    
    # define the serialization instructions for each PRAW object type
    instruction = {
        praw.models.Submission: { 
            'function': _serialize_submission,
            'cache_key_attribute': None,
        },
        praw.models.Comment: {
            'function': _serialize_comment,
            'cache_key_attribute': None,
        },
        praw.models.Subreddit: {
            'function': _serialize_subreddit,
            'cache_key_attribute': None,
        },
        praw.models.Redditor: {
            'function': _serialize_redditor,
            'cache_key_attribute': 'name',
        },
        praw.models.RemovalReason: {
            'function': _serialize_removal_reason,
            'cache_key_attribute': None,
        },
        praw.models.Rule: {
            'function': _serialize_rule,
            'cache_key_attribute': None,
        },
        praw.models.PollOption: {
            'function': _serialize_poll_option,
            'cache_key_attribute': None,
        },
        praw.models.PollData: {
            'function': _serialize_poll_data,
            'cache_key_attribute': None,
        },
        praw.models.reddit.subreddit.SubredditRedditorFlairTemplates: {
            'function': _serialize_subreddit_flair_templates,
            'cache_key_attribute': None,
        },
        praw.models.ButtonWidget: {
            'function': _serialize_button_widget,
            'cache_key_attribute': None,
        },
        praw.models.Button: {
            'function': _serialize_button,
            'cache_key_attribute': None,
        },
        praw.models.Calendar: {
            'function': _serialize_calendar,
            'cache_key_attribute': None,
        },
        praw.models.CommunityList: {
            'function': _serialize_community_list,
            'cache_key_attribute': None,
        },
        praw.models.CustomWidget: {
            'function': _serialize_custom_widget,
            'cache_key_attribute': None,
        },
        praw.models.IDCard: {
            'function': _serialize_id_card,
            'cache_key_attribute': None,
        },
        praw.models.ImageWidget: {
            'function': _serialize_image_widget,
            'cache_key_attribute': None,
        },
        praw.models.ImageData: {
            'function': _serialize_image,
            'cache_key_attribute': None,
        },
        praw.models.Image: {
            'function': _serialize_image,
            'cache_key_attribute': None,
        },
        praw.models.ModeratorsWidget: {
            'function': _serialize_moderators_widget,
            'cache_key_attribute': None,
        },
        praw.models.PostFlairWidget: {
            'function': _serialize_post_flair_widget,
            'cache_key_attribute': None,
        },
        praw.models.RulesWidget: {
            'function': _serialize_rules_widget,
            'cache_key_attribute': None,
        },
        praw.models.TextArea: {
            'function': _serialize_text_area,
            'cache_key_attribute': None,
        },
        dict: {
            'function': _serialize_dict,
            'cache_key_attribute': None,
        },
    }[type(obj)]
    
    function = instruction['function']
    cache_key_attribute = instruction['cache_key_attribute']
    
    if cache_key_attribute is not None:
        type_string = type(obj).__name__
        attribute_value = getattr(obj, cache_key_attribute)
        cache_key = f"{type_string}_{attribute_value}"
    else:
        cache_key = None
    
    # check if the object is already in the cache
    if cache_key is not None and cache_key in cache:
        # return the cached object
        return cache[cache_key]
    
    # serialize the object
    data = function(obj, cache=cache)
    
    # add the object to the cache
    if cache_key is not None:
        cache[cache_key] = data
        
    # return the serialized object
    return data

def _serialize_dict(o: dict, cache : Cache = None):
    return o

def _serialize_submission(o: praw.models.Submission, cache : Cache = None):
    # https://praw.readthedocs.io/en/stable/code_overview/models/submission.html
    # if the submission is a poll, the poll_data attribute will be a PollData object
    # try:
    #     poll_data = serialize(o.poll_data)
    # except AttributeError:
    #     poll_data = None
    
    # if hasattr(o, "link_flair_template_id"):
    #     link_flair_template_id = o.link_flair_template_id
    # else:
    #     link_flair_template_id = None

    return {
        "author": serialize(o.author, cache=cache),
        "author_flair_text": o.author_flair_text,
        "created_utc": o.created_utc,
        "distinguished": o.distinguished,
        "edited": o.edited,
        "id": o.id,
        "is_original_content": o.is_original_content,
        "is_self": o.is_self,
        "link_flair_template_id": None, # this calls the reddit api so we don't want to do this
        "link_flair_text": "", # this calls the reddit API, so we don't want to do this
        "locked": o.locked,
        "name": o.name,
        "num_comments": o.num_comments,
        "over_18": o.over_18,
        "permalink": o.permalink,
        # "poll_data": poll_data,
        "score": o.score,
        "selftext": o.selftext,
        "spoiler": o.spoiler,
        "stickied": o.stickied,
        "title": o.title,
        "upvote_ratio": o.upvote_ratio,
        "url": o.url,
        "user_reports": serialize_list(o.user_reports),
    }

def _serialize_comment(o: praw.models.Comment, is_report : bool = False, cache : Cache = None):
    # https://praw.readthedocs.io/en/stable/code_overview/models/comment.html
    data = {
        "author": serialize(o.author, cache=cache),
        "body": o.body,
        "body_html": o.body_html,
        "created_utc": o.created_utc,
        "distinguished": o.distinguished,
        "edited": o.edited,
        "id": o.id,
        "is_submitter": o.is_submitter,
        "link_id": o.link_id,
        "parent_id": o.parent_id,
        "permalink": o.permalink,
        "score": o.score,
        "stickied": o.stickied,
        "subreddit_id": o.subreddit_id,
        "user_reports": serialize_list(o.user_reports, cache=cache),
    }
    
    return data

def _serialize_subreddit(o: praw.models.Subreddit, cache : Cache = None):
    # https://praw.readthedocs.io/en/stable/code_overview/models/subreddit.html
    return {
        "can_assign_link_flair": o.can_assign_link_flair,
        "can_assign_user_flair": o.can_assign_user_flair,
        "created_utc": o.created_utc,
        "description": o.description,
        "description_html": o.description_html,
        "display_name": o.display_name,
        "flair_templates": _serialize_subreddit_flair_templates(o.flair.templates, cache=cache),
        "id": o.id,
        "name": o.name,
        "over18": o.over18,
        "public_description": o.public_description,
        "spoilers_enabled": o.spoilers_enabled,
        "subscribers": o.subscribers,
    }

def _serialize_redditor(o: praw.models.Redditor, cache : Cache = None):
    # https://praw.readthedocs.io/en/stable/code_overview/models/redditor.html

    # if o responds to is_suspended, and it is true
    if hasattr(o, "is_suspended") and o.is_suspended:
        return {
            "name": o.name,
            "is_suspended": o.is_suspended,
            "awardee_karma": o.awardee_karma,
            "awarder_karma": o.awarder_karma,
            "is_blocked": o.is_blocked,
            "total_karma": o.total_karma,
        }

    return {
        "comment_karma": o.comment_karma,
        "total_karma": o.total_karma,
        "created_utc": o.created_utc,
        "has_verified_email": o.has_verified_email,
        "icon_img": o.icon_img,
        "id": o.id,
        "is_employee": o.is_employee,
        "is_mod": o.is_mod,
        "is_gold": o.is_gold,
        "name": o.name,
    }

def _serialize_rule(o: praw.models.Rule, cache : Cache = None):
    # https://praw.readthedocs.io/en/stable/code_overview/other/rule.html
    return {
        "created_utc": o.created_utc,
        "description": o.description,
        "kind": o.kind,
        "priority": o.priority,
        "short_name": o.short_name,
        "violation_reason": o.violation_reason,
    }

def _serialize_removal_reason(o: praw.models.RemovalReason, cache : Cache = None):
    # https://praw.readthedocs.io/en/stable/code_overview/other/removalreason.html
    return {
        "id": o.id,
        "message": o.message,
        "title": o.title,
    }

def _serialize_poll_option(o: praw.models.PollOption, cache : Cache = None):
    # https://praw.readthedocs.io/en/stable/code_overview/other/polloption.html
    return {
        "id": o.id,
        "text": o.text,
        "vote_count": o.vote_count,
    }

def _serialize_poll_data(o: praw.models.PollData, cache : Cache = None):
    # https://praw.readthedocs.io/en/stable/code_overview/other/polldata.html
    return {
        "options": [serialize(option) for option in o.options],
        "total_vote_count": o.total_vote_count,
        "voting_end_timestamp": o.voting_end_timestamp,
    }

def _serialize_subreddit_flair_templates(o: praw.models.reddit.subreddit.SubredditRedditorFlairTemplates, cache : Cache = None):
    # https://praw.readthedocs.io/en/stable/code_overview/other/subredditredditorflairtemplates.html
    templates = []
    for template in o:
        templates.append(_serialize_subreddit_flair_template(template))
    return templates

def _serialize_button_widget(o: praw.models.ButtonWidget, cache : Cache = None):
    return {
        "buttons": serialize_list(o.buttons, cache=cache),
        "description": o.description,
        "description_html": None, # o.description_html, # Apparently this is no longer an attribute from API
        "id": o.id,
        "kind": o.kind,
        "short_name": o.shortName,
        "styles": o.styles,
    }
    
def _serialize_button(o: praw.models.Button, cache : Cache = None):
    return {
        "color": o.color,
        "fill_color": o.fillColor,
        "height": o.height if hasattr(o, "height") else None,
        "hover_state": o.hoverState if hasattr(o, "hoverState") else None,
        "kind": o.kind,
        "link_url": o.linkUrl if hasattr(o, "linkUrl") else None,
        "text": o.text,
        "text_color": o.textColor,
        "url": o.url,
        "width": o.width if hasattr(o, "width") else None,
    }

def _serialize_calendar(o: praw.models.Calendar, cache : Cache = None):
    return {
        "configuration": o.configuration,
        "data": o.data,
        "id": o.id,
        "kind": o.kind,
        "requires_sync": o.requiresSync,
        "short_name": o.shortName,
        "styles": o.styles,
    }

def _serialize_community_list(o: praw.models.CommunityList, cache : Cache = None):
    return {
        "data": _serialize_subreddit_list(o.data, cache=cache),
        "id": o.id,
        "kind": o.kind,
        "short_name": o.shortName,
        "styles": o.styles,
    }

def _serialize_custom_widget(o: praw.models.CustomWidget, cache : Cache = None):
    return {
        "css": o.css,
        "height": o.height,
        "id": o.id,
        "imageData": serialize_list(o.imageData, cache=cache),
        "kind": o.kind,
        "short_name": o.shortName,
        "styles": o.styles,
        "stylesheet_url": o.stylesheetUrl,
        "text": o.text,
        "text_html": o.textHtml,
    }

def _serialize_id_card(o: praw.models.IDCard, cache : Cache = None):
    return {
        "currently_viewing_count": o.currentlyViewingCount,
        "currently_viewing_text": o.currentlyViewingText,
        "description": o.description,
        "id": o.id,
        "kind": o.kind,
        "short_name": o.shortName,
        "styles": o.styles,
        "subscribers_count": o.subscribersCount,
        "subscribers_text": o.subscribersText,
    }

def _serialize_image_widget(o: praw.models.ImageWidget, cache : Cache = None):
    return {
        "data": serialize_list(o.data, cache=cache),
        "id": o.id,
        "kind": o.kind,
        "short_name": o.shortName,
        "styles": o.styles,
    }

def _serialize_image(o: praw.models.ImageData | praw.models.Image, cache : Cache = None):
    return {
        "height": o.height,
        "link_url": o.linkUrl,
        "url": o.url,
        "width": o.width,
    }

def _serialize_moderators_widget(o: praw.models.ModeratorsWidget, cache : Cache = None):
    return {
        "id": o.id,
        "kind": o.kind,
        "mods": serialize_list(o.mods, cache=cache),
        "short_name": o.shortName,
        "styles": o.styles,
    }

def _serialize_post_flair_widget(o: praw.models.PostFlairWidget, cache : Cache = None):
    return {
        "display": o.display,
        "id": o.id,
        "kind": o.kind,
        "order": o.order,
        "short_name": o.shortName,
        "styles": o.styles,
        "templates": o.templates
    }

def _serialize_rules_widget(o: praw.models.RulesWidget, cache : Cache = None):
    return {
        "data": serialize_list(o.data, cache=cache),
        "display": o.display,
        "id": o.id,
        "kind": o.kind,
        "short_name": o.shortName,
        "styles": o.styles,
    }

def _serialize_text_area(o: praw.models.TextArea, cache : Cache = None):
    return {
        "id": o.id,
        "kind": o.kind,
        "short_name": o.shortName,
        "styles": o.styles,
        "text": o.text,
        "text_html": o.textHtml,
    }
    
def _serialize_subreddit_list(o: list[praw.models.Subreddit], cache : Cache = None):
    # extract ids from the list of subreddits
    return [subreddit.id for subreddit in o]

def _serialize_subreddit_flair_template(o: dict):
    return o



"""
vegmod.pycall exposes simple functions that can be called from the Ruby language.
Each function must only accept and return simple types (numeric, string, arrays, hashes, etc.)
"""
import os
import requests
from time import sleep
from loguru import logger
from PIL import Image
from typing import Optional, Dict, List
from vegmod import reddit

def comment_delete(comment_id : str) -> None:
    """
    Delete a comment by ID.
    """
    return reddit.comment(comment_id).delete()

def comment_edit(comment_id : str, body : str) -> None:
    """
    Edit a comment by ID.
    """
    return reddit.comment(comment_id).edit(body)

def comment_mod_approve(comment_id : str) -> None:
    """
    Approve a comment by ID.
    """
    return reddit.comment(comment_id).mod.approve()

def comment_mod_note(comment_id : str, note : str) -> None:
    """
    Create a mod note on a comment.
    """
    return reddit.comment(comment_id).mod.note(note)

def comment_mod_distinquish(comment_id : str, how : str = 'yes', sticky: bool = False) -> None:
    """
    Distinguish a comment by ID.
    
    how can be "yes", "no", "admin", or "special".
    """
    return reddit.comment(comment_id).mod.distinguish(how=how, sticky=sticky)

def comment_mod_ignore_reports(comment_id : str) -> None:
    """
    Ignore reports on a comment.
    """
    return reddit.comment(comment_id).mod.ignore_reports()

def comment_mod_lock(comment_id : str) -> None:
    """
    Lock a comment by ID.
    """
    return reddit.comment(comment_id).mod.lock()

def comment_mod_remove(comment_id : str, mod_note : str = '', spam: bool = False, reason_id: str | None = None) -> None:
    """
    Remove a comment by ID.
    """
    return reddit.comment(comment_id).mod.remove(mod_note=mod_note, spam=spam, reason_id=reason_id)

def comment_mod_send_removal_message(comment_id : str, message: str) -> None:
    """
    Send a removal message to the author of a comment.
    """
    return reddit.comment(comment_id).mod.send_removal_message(message=message)

def comment_mod_undistinguish(comment_id : str) -> None:
    """
    Undistinguish a comment by ID.
    """
    return reddit.comment(comment_id).mod.undistinguish()

def comment_mod_unignore_reports(comment_id : str) -> None:
    """
    Unignore reports on a comment.
    """
    return reddit.comment(comment_id).mod.unignore_reports()

def comment_mod_unlock(comment_id : str) -> None:
    """
    Unlock a comment by ID.
    """
    return reddit.comment(comment_id).mod.unlock()

def comment_report(comment_id : str, reason : str) -> None:
    """
    Report a comment by ID.
    """
    return reddit.comment(comment_id).report(reason)

def comment_reply(comment_id : str, body : str) -> str:
    """
    Reply to a comment with a reply.
    """
    return reddit.comment(comment_id).reply(body)

def comment_reply_distinguish_lock(comment_id : str, body : str, how : str = 'yes', sticky: bool = False) -> str:
    """
    Reply to a comment with a reply, distinguish it, and lock it.
    """
    comment = reddit.comment(comment_id).reply(body)
    comment.mod.distinguish(how=how, sticky=sticky)
    comment.mod.lock()
    return comment

def submission_delete(submission_id : str) -> None:
    """
    Delete a submission by ID.
    """
    return reddit.submission(submission_id).delete()

def submission_edit(submission_id : str, body : str) -> None:
    """
    Edit a submission by ID.
    """
    return reddit.submission(submission_id).edit(body)

def submission_mod_approve(submission_id : str) -> None:
    """
    Approve a submission by ID.
    """
    return reddit.submission(submission_id).mod.approve()

def submission_mod_create_note(submission_id : str, label : str, note : str) -> None:
    """
    Create a mod note on a submission.
    """
    return reddit.submission(submission_id).mod.create_note(label=label, note=note)

def submission_mod_distinguish(submission_id : str, how : str = 'yes', sticky: bool = False) -> None:
    """
    Distinguish a submission by ID.
    
    how can be "yes", "no", "admin", or "special".
    """
    return reddit.submission(submission_id).mod.distinguish(how=how, sticky=sticky)

def submission_mod_flair(submission_id : str, flair_template_id : str | None = None, text : str = '') -> None:
    """
    Set the flair on a submission.
    """
    return reddit.submission(submission_id).mod.flair(flair_template_id=flair_template_id, text=text)

def submission_mod_ignore_reports(submission_id : str) -> None:
    """
    Ignore reports on a submission.
    """
    return reddit.submission(submission_id).mod.ignore_reports()

def submission_mod_lock(submission_id : str) -> None:
    """
    Lock a submission by ID.
    """
    return reddit.submission(submission_id).mod.lock()

def submission_mod_nsfw(submission_id : str) -> None:
    """
    Mark a submission as NSFW.
    """
    return reddit.submission(submission_id).mod.nsfw()

def submission_mod_remove(submission_id : str, mod_note : str = '', spam: bool = False, reason_id: str | None = None) -> None:
    """
    Remove a submission by ID.
    """
    return reddit.submission(submission_id).mod.remove(mod_note=mod_note, spam=spam, reason_id=reason_id)

def submission_reply(submission_id : str, body : str) -> str:
    """
    Reply to a submission with a reply.
    """
    return reddit.submission(submission_id).reply(body)

def submission_mod_send_removal_message(submission_id : str, message: str) -> None:
    """
    Send a removal message to the author of a submission.
    """
    return reddit.submission(submission_id).mod.send_removal_message(message=message)

def submission_mod_sfw(submission_id : str) -> None:
    """
    Mark a submission as SFW.
    """
    return reddit.submission(submission_id).mod.sfw()

def submission_mod_spoiler(submission_id : str) -> None:
    """
    Mark a submission as a spoiler.
    """
    return reddit.submission(submission_id).mod.spoiler()

def submission_mod_sticky(submission_id : str, bottom: bool = True, state: bool = True) -> None:
    """
    Sticky a submission by ID.
    """
    return reddit.submission(submission_id).mod.sticky(bottom=bottom, state=state)

def submission_mod_suggested_sort(submission_id : str, sort : str = 'blank') -> None:
    """
    Set the suggested sort on a submission.
    """
    return reddit.submission(submission_id).mod.suggested_sort(sort=sort)

def submission_mod_undistinguish(submission_id : str) -> None:
    """
    Undistinguish a submission by ID.
    """
    return reddit.submission(submission_id).mod.undistinguish()

def submission_mod_unignore_reports(submission_id : str) -> None:
    """
    Unignore reports on a submission.
    """
    return reddit.submission(submission_id).mod.unignore_reports()

def submission_mod_unlock(submission_id : str) -> None:
    """
    Unlock a submission by ID.
    """
    return reddit.submission(submission_id).mod.unlock()

def submission_mod_unspoiler(submission_id : str) -> None:
    """
    Unmark a submission as a spoiler.
    """
    return reddit.submission(submission_id).mod.unspoiler()

def submission_mod_update_crowd_control_level(submission_id : str, level : int) -> None:
    """
    Update the crowd control level on a submission.
    """
    return reddit.submission(submission_id).mod.update_crowd_control_level(level=level)

def submission_report(submission_id : str, reason : str) -> None:
    """
    Report a submission by ID.
    """
    return reddit.submission(submission_id).report(reason)

def subreddit_contributor_add(subreddit_id : str, redditor_id : str) -> None:
    """
    Add a contributor to a subreddit.
    """
    return reddit.subreddit(subreddit_id).contributor.add(redditor_id)

def subreddit_contributor_exists(subreddit_id : str, redditor_id : str) -> bool:
    """
    Check if a redditor is a contributor to a subreddit.
    """
    return reddit.subreddit(subreddit_id).contributor(redditor_id) is not None

def subreddit_contributor_remove(subreddit_id : str, redditor_id : str) -> None:
    """
    Remove a contributor from a subreddit.
    """
    return reddit.subreddit(subreddit_id).contributor.remove(redditor_id)

def subreddit_mod_settings(subreddit_id) -> dict[str, str | int | bool]:
    """
    Get the settings on a subreddit.
    """
    return reddit.subreddit(subreddit_id).mod.settings()

def subreddit_mod_update(subreddit_id : str, **settings : str | int | bool) -> dict[str, str | int | bool]:
    """
    Update the settings on a subreddit.
    """
    return reddit.subreddit(subreddit_id).mod.update(**settings)

def subreddit_widgets_sidebar_delete_all(subreddit_id : str) -> None:
    """
    Delete all sidebar widgets on a subreddit.
    """
    try:
        for widget in reddit.subreddit(subreddit_id).widgets.sidebar:
            permanent_kinds = [
                'id-card'
                'moderators',
                'subreddit-rules',
            ]
            if widget.kind not in permanent_kinds:       
                try:
                    # request will silently fail if not debounced
                    sleep(3)

                    widget.mod.delete()
                except Exception as e:
                    logger.error(f"Error deleting sidebar widget: {widget}, e: {e}")
    except Exception as e:
        logger.error(f"Error deleting sidebar widgets: {e}")
    finally:
        return None

def subreddit_widgets_sidebar_delete_widget(subreddit_id: str, widget_id: str) -> None:
    """
    Delete a sidebar widget on a subreddit.
    """
    try:
        for widget in reddit.subreddit(subreddit_id).widgets.sidebar:
            if widget.id == widget_id:
                widget.mod.delete()
    except Exception as e:
        logger.error(f"Error deleting sidebar widget: {widget_id}, e: {e}")
    finally:
        return None

def subreddit_widgets_mod_upload_image(subreddit_id: str, image_url: str, link_url: str) -> Optional[Dict]:
    if '.' not in image_url:
        logger.error(f"Invalid image URL: {image_url}")
        return None

    image_url_ext = image_url.split('.')[-1]
    if image_url_ext not in ['png', 'jpg', 'jpeg']:
        logger.error(f"Invalid image EXT: {image_url}")
        return None
    
    # save the image to a file
    image_file_path = f'/tmp/praw_image_upload.{image_url_ext}'
    try:
        response = requests.get(image_url)
        response.raise_for_status()
        with open(image_file_path, 'wb') as f:
            f.write(response.content)
        
        # extract the image dimensions using PIL
        with Image.open(image_file_path) as img:
            width, height = img.size
        
        upload_url = reddit.subreddit(
            subreddit_id
        ).widgets.mod.upload_image(
            image_file_path
        )
        
        return {
            'url': upload_url,
            'linkUrl': link_url,
            'width': width,
            'height': height,
        }
    except requests.RequestException as e:
        logger.error(f"Error downloading image: {e}")
        return None
    except IOError as e:
        logger.error(f"Error processing image file: {e}")
        return None
    finally:
        # Clean up the temporary image file
        if os.path.exists(image_file_path):
            os.remove(image_file_path)
        

def subreddit_widgets_mod_add_image_widget(subreddit_id: str, short_name: str, image_url: str, link_url: str) -> Optional[Dict]:
    """
    Add an image widget to a subreddit.
    """
    try:
        image_data = [
            subreddit_widgets_mod_upload_image(subreddit_id, image_url, link_url)
        ]
        
        styles = {
            'backgroundColor': '#FFFFFF',
            'headerColor': '#0079d3',
        }
        
        widget = reddit.subreddit(
            subreddit_id
        ).widgets.mod.add_image_widget(
            short_name=short_name, 
            data=image_data, 
            styles=styles,
            link_url=link_url
        )
        
        return widget
    except Exception as e:
        logger.error(f"Error adding image widget: {e}")
        return None

def subreddit_widgets_mod_reupload_image(subreddit_id: str, widget_id: str, image_url: str, link_url: str) -> Optional[Dict]:
    """
    Update an image widget on a subreddit.
    """
    try:
        widget = None
        for w in reddit.subreddit(subreddit_id).widgets.sidebar:
            if w.id == widget_id:
                widget = w
                break
        
        if widget is None:
            logger.error(f"Widget not found: {widget_id}")
            return None
        
        image_data = [
            subreddit_widgets_mod_upload_image(subreddit_id, image_url, link_url)
        ]
        
        widget = widget.mod.update(
            data=image_data, 
        )
        
        return widget
    except Exception as e:
        logger.error(f"Error updating image widget: {e}")
        return None

def subreddit_widgets_mod_add_community_list(subreddit_id: str, short_name: str, description: str, subreddits: List[str]) -> Optional[Dict]:
    """
    Add a community widget to a subreddit.

    Parameters:
    subreddit_id (str): The ID of the subreddit where the widget will be added.
    short_name (str): The short name for the widget.
    description (str): The description for the widget.
    subreddits (List[str]): A list of subreddit names to include in the community widget.

    Returns:
    Optional[Dict]: The response from the Reddit API if successful, None otherwise.
    """
    try:
        styles = {
            'backgroundColor': '#FFFFFF',
            'headerColor': '#0079d3',
        }
                
        return reddit.subreddit(
            subreddit_id
        ).widgets.mod.add_community_list(
            short_name=short_name, 
            description=description,
            data=subreddits,
            styles=styles
        )
    except Exception as e:
        logger.error(f"Error adding community widget: {e}")
        return None

def subreddit_widgets_mod_add_button_widget(subreddit_id: str, short_name: str, description: str, texts: List[str], urls: List[str]) -> Optional[Dict]:
    """
    Add a button widget to a subreddit.

    Parameters:
    subreddit_id (str): The ID of the subreddit where the widget will be added.
    short_name (str): The short name for the widget.
    description (str): The description for the widget.
    texts (List[str]): A list of button texts.
    urls (List[str]): A list of URLs to link to when the buttons are clicked.

    Returns:
    Optional[Dict]: The response from the Reddit API if successful, None otherwise.
    """    
    try:
        styles = {
            'backgroundColor': '#FFFFFF',
            'headerColor': '#0079d3',
        }
                
        buttons = [
            {
                'kind': 'text',
                'text': text,
                'url': url,
                'color': '#FF4500',
                'fillColor': '#FFFFFF',
                'textColor': '#000000',
            }
            for text, url in zip(texts, urls)
        ]
        return reddit.subreddit(
            subreddit_id
        ).widgets.mod.add_button_widget(
            short_name=short_name, 
            description=description,
            buttons=buttons,
            styles=styles
        ).id
    except Exception as e:
        logger.error(f"Error adding button widget: {e}")
        return None

def subreddit_widgets_mod_reorder(subreddit_id: str, widget_ids: List[str]) -> None:
    """
    Reorder widgets on a subreddit.

    Parameters:
    subreddit_id (str): The ID of the subreddit where the widgets will be reordered.
    widget_ids (List[str]): A list of widget IDs in the order they should be displayed.
    """
    try:
        # Get the existing widget order
        existing_order = [widget.id for widget in reddit.subreddit(subreddit_id).widgets.sidebar]
        new_order = []
        
        # Add the new widget IDs in the order they were provided
        for widget_id in widget_ids:
            # Only add the widget ID if it exists
            if widget_id in existing_order:
                new_order.append(widget_id)
        
        # Add any existing widget IDs that weren't provided
        for widget_id in existing_order:
            if widget_id not in new_order:
                new_order.append(widget_id)
        
        # Reorder the widgets
        reddit.subreddit(subreddit_id).widgets.mod.reorder(new_order)
    except Exception as e:
        logger.error(f"Error reordering widgets: {e}")
        return None


import time

from loguru import logger
from vegmod.utils import save_dict, load_dict

class Cache:
    def __init__(self, file_path: str):
        self.file_path = file_path
        
        # remove any entries that are older than 1 hour
        dirty_cache = load_dict(file_path)
        self.cache = {k: v for k, v in dirty_cache.items() if time.time() - v["_cached_at"] < 3600}

    def save(self):
        """
        Save the cache to a JSON file.
        """
        save_dict(self.cache, self.file_path)
    
    # override get and set [] operators
    def __getitem__(self, key):
        # logger.info(f"Cache Get: key={key}")
        return self.cache[key]['data']
    
    # override set [] operator
    def __setitem__(self, key, value):
        # logger.info(f"Cache Set: key={key}")
        self.cache[key] = {
            'data': value,
            '_cached_at': time.time()     
        }
        
    # override 'in' operator
    def __contains__(self, key):
        return key in self.cache
    
    # override 'del' operator
    def __delitem__(self, key):
        del self.cache[key]
        
    # override 'len' operator
    def __len__(self):
        return len(self.cache)
    
    # override items() method
    def items(self):
        return self.cache.items()


import json
from loguru import logger

def save_dict(data: dict, file_path: str):
    """
    Save the data to a JSON file.
    """
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(json.dumps(data, indent=4))
    
def load_dict(file_path: str) -> dict:
    """
    Load the data from a JSON file.
    """
    try:
        with open(file_path, "r", encoding="utf-8") as f:
            return json.loads(f.read())
    except FileNotFoundError:
        return {}
    except Exception as e:
        logger.error(f"Failed to load file={file_path} error={e}")
        return {}